@@ -53,6 +53,75 @@ class CustomerOrderBatchHandler(BatchHandler):
     has_custom_product_autocomplete = False
     nondigits_pattern = re.compile(r'\D')
 
+    def init_batch(self, batch, progress=None, **kwargs):
+        """
+        Assign the "local" store to the batch, if applicable.
+        """
+        session = self.app.get_session(batch)
+        batch.store = self.config.get_store(session)
+
+    def new_order_requires_customer(self):
+        """
+        Returns a boolean indicating whether a *new* "customer order"
+        in fact requires a proper customer account, or not.  Note that
+        in all cases a new order requires a *person* to associate
+        with, but technically the customer is optional, unless this
+        returns true.
+        """
+        return self.config.getbool('rattail.custorders',
+                                   'new_order_requires_customer',
+                                   default=False)
+
+    def assign_contact(self, batch, customer=None, person=None, **kwargs):
+        """
+        Assign the customer and/or person "contact" for the order.
+        """
+        clientele = self.app.get_clientele_handler()
+        customer_required = self.new_order_requires_customer()
+
+        # nb. person is always required
+        if customer and not person:
+            person = clientele.get_person(customer)
+        if not person:
+            raise ValueError("Must specify a person")
+
+        # customer may or may not be optional
+        if person and not customer:
+            customer = clientele.get_customer(person)
+        if customer_required and not customer:
+            raise ValueError("Must specify a customer account")
+
+        # assign contact
+        batch.customer = customer
+        batch.person = person
+
+        # update phone/email per new contact
+        batch.phone_number = None
+        batch.email_address = None
+        if customer_required:
+            batch.phone_number = clientele.get_first_phone_number(customer)
+            batch.email_address = clientele.get_first_email_address(customer)
+        else:
+            batch.phone_number = person.first_phone_number()
+            batch.email_address = person.first_email_address()
+
+        session = self.app.get_session(batch)
+        session.flush()
+        session.refresh(batch)
+
+    def unassign_contact(self, batch, **kwargs):
+        """
+        Unassign the customer and/or person "contact" for the order.
+        """
+        batch.customer = None
+        batch.person = None
+        batch.phone_number = None
+        batch.email_address = None
+
+        session = self.app.get_session(batch)
+        session.flush()
+        session.refresh(batch)
+
     def get_case_size_for_product(self, product):
         if product.case_size:
             return product.case_size
@@ -72,8 +141,8 @@ class CustomerOrderBatchHandler(BatchHandler):
 
     def customer_autocomplete(self, session, term, **kwargs):
         """
-        Override the customer autocomplete, to search by phone number
-        as well as customer name.
+        Override the Customer autocomplete, to search by phone number
+        as well as name.
         """
         model = self.model
 
@@ -94,7 +163,7 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         else: # term does not look like a phone number
 
-            # so just search by customer name
+            # so just search by name
             criteria = [model.Customer.name.ilike('%{}%'.format(word))
                         for word in term.split()]
             query = query.filter(sa.and_(*criteria))
@@ -116,6 +185,85 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         return results
 
+    def person_autocomplete(self, session, term, **kwargs):
+        """
+        Override the Person autocomplete, to search by phone number as
+        well as name.
+        """
+        model = self.model
+
+        # define the base query
+        query = session.query(model.Person)\
+                       .options(orm.joinedload(model.Person.phones))
+
+        # does search term look like a phone number?
+        phone_term = self.get_phone_search_term(term)
+        if phone_term:
+
+            # yep, so just search for the phone number
+            query = query.join(model.PersonPhoneNumber,
+                               model.PersonPhoneNumber.parent_uuid == model.Person.uuid)
+            query = query.filter(sa.func.regexp_replace(model.PersonPhoneNumber.number,
+                                                        r'\D', '', 'g')\
+                                 .like('%{}%'.format(phone_term)))
+
+        else: # term does not look like a phone number
+
+            # so just search by name
+            criteria = [model.Person.display_name.ilike('%{}%'.format(word))
+                        for word in term.split()]
+            query = query.filter(sa.and_(*criteria))
+
+        # oh, and sort by something useful
+        query = query.order_by(model.Person.display_name)
+
+        # generate result list from query
+        results = []
+        for person in query:
+            phone = person.first_phone()
+            if phone:
+                label = "{} {}".format(person.display_name, phone.number)
+            else:
+                label = person.display_name
+            results.append({'value': person.uuid,
+                            'label': label,
+                            'display': person.display_name})
+
+        return results
+
+    def get_customer_info(self, batch, **kwargs):
+        """
+        Return a data dict containing misc. info pertaining to the
+        customer/person for the order batch.
+        """
+        info = {
+            'customer_uuid': None,
+            'person_uuid': None,
+            'phone_number': None,
+            'email_address': None,
+        }
+
+        if batch.customer:
+            info['customer_uuid'] = batch.customer.uuid
+            phone = batch.customer.first_phone()
+            if phone:
+                info['phone_number'] = phone.number
+            email = batch.customer.first_email()
+            if email:
+                info['email_address'] = email.address
+
+        if batch.person:
+            info['person_uuid'] = batch.person.uuid
+            if not info['phone_number']:
+                phone = batch.person.first_phone()
+                if phone:
+                    info['phone_number'] = phone.number
+                email = batch.person.first_email()
+                if email:
+                    info['email_address'] = email.address
+
+        return info
+
     def custom_product_autocomplete(self, session, term, **kwargs):
         """
         For the given term, this should return a (possibly empty) list
@@ -232,7 +380,7 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         def convert(row, i):
             item = model.CustomerOrderItem()
-            item.sequence = i + 1
+            item.sequence = i
             item.status_code = self.enum.CUSTORDER_ITEM_STATUS_ORDERED
             for field in row_fields:
                 setattr(item, field, getattr(row, field))
