@@ -39,9 +39,11 @@ from openpyxl.cell.cell import get_column_letter
 from rattail.util import progress_loop
 
 
-class ExcelReader(object):
+class ExcelReaderXLS(object):
     """
-    Basic class for reading Excel files.
+    Basic class for reading Excel "legacy" (.xls) files.
+
+    Uses the ``xlrd`` package to read the files.
     """
 
     def __init__(self, path, sheet=0, sheet_name=None, header=0,
@@ -99,6 +101,56 @@ class ExcelReader(object):
         if value:
             return datetime.datetime.strptime(value, fmt or self.datefmt).date()
 
+# TODO: this should become a base class and/or wrapper of some sort. for now
+# the "default" reader assumes XLS since apparently that is all we have ever
+# supported until now...which was a surprise to me...
+ExcelReader = ExcelReaderXLS
+
+
+class ExcelReaderXLSX(object):
+    """
+    Basic class for reading Excel 2010 (.xslx) files.
+
+    Uses the ``openpyxl`` package to read the files.
+    """
+
+    def __init__(self, path, header_row=1, **kwargs):
+        """
+        Constructor; opens an Excel file for reading.
+
+        :param path: Path to the Excel file.
+
+        :param header_row: Which row contains the column headers.  This is
+           1-based, so the 1 is the default (i.e. the first row).
+        """
+        self.book = openpyxl.load_workbook(filename=path)
+        self.sheet = self.book.active
+
+        self.header_row = header_row
+
+        self.fields = None
+        # TODO: this seems like a hacky way to get the header fields?  we are
+        # "iterating" over the single header row, effectively...
+        for row in self.sheet.iter_rows(min_row=self.header_row,
+                                        max_row=self.header_row,
+                                        values_only=True):
+            assert self.fields is None
+            self.fields = list(row)
+
+    def read_rows(self, progress=None):
+        rows = []
+
+        def append(row, i):
+            data = dict([(self.fields[j], value)
+                         for j, value in enumerate(row)])
+            rows.append(data)
+
+        xlrows = list(self.sheet.iter_rows(min_row=self.header_row + 1,
+                                           values_only=True))
+        progress_loop(append, xlrows, progress,
+                      message="Reading data from Excel file")
+        return rows
+
 
 class ExcelWriter(object):
     """
